package com.jh.boot.web;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.jh.boot.jpa.AbstractIdEntity;
import com.jh.boot.web.list.Page;
import com.jh.boot.web.list.SortTrend;

public class Converter {

    private Converter() {}

    public static <S, D> D convert(S source, Function<S, D> mappingFunction) {
        if (source == null || mappingFunction == null) {
            return null;
        }

        return mappingFunction.apply(source);
    }

    public static <S, D> Set<D> convertSet(Collection<S> list, Function<S, D> mappingFunction) {
    	if (list == null) {
    		return null;
    	}
    	return list.stream().map(mappingFunction).collect(Collectors.toSet());
    }
    
    public static <S, D> List<D> convertList(Collection<S> list, Function<S, D> mappingFunction) {
    	if (list == null) {
    		return null;
    	}
    	return list.stream().map(mappingFunction).collect(Collectors.toList());
    }
    
    public static <S, D> Page<D> convertPage(Page<S> page, Function<S, D> mappingFunction) {
        if (page == null || mappingFunction == null) {
            return null;
        }

        List<D> content = new ArrayList<>();
        if (page.getContent() != null) {
            for (S s : page.getContent()) {
                content.add(mappingFunction.apply(s));
            }
        }

        return new Page<D>(page.getPage(), page.getPagesCount(), page.getPageSize(), page.getTotalElements(), content, page.getSortBy(), SortTrend.parse(page.getTrend()));
    }
    
    
    public static <T> List<IdNameDto> mapToDtos(Collection<T> collection, Function<T, Long> getId, Function<T, String> getName) {
    	if (collection == null || collection.isEmpty() || getId == null || getName == null) {
    		return Collections.emptyList();
    	}
    	
    	List<IdNameDto> result = new ArrayList<IdNameDto>();
    	for (T t : collection) {
    		result.add(new IdNameDto(getId.apply(t), getName.apply(t)));
    	}
    	return result;
    }

    //public static <T> T toEntity(IdNameDto )
    
    /**
     * Helper function to safely get id
     *
     * @param entity
     * @return
     */
    public static Long getId(AbstractIdEntity entity) {
        return entity == null ? null : entity.getId();
    }

}
